package ac.ed.lurg.country.gams;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.GlobalPrice;
import ac.ed.lurg.country.ImportExportConstraint;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.CropType;

public class GamsCountryInput {

	private CompositeCountry country; // not really required but useful for debugging
	private Map<CommodityType, Double> projectedDemand;
	private Map<CropType, GlobalPrice> worldPrices;
	private Map<CropType, ImportExportConstraint> importConstraints;
	private Map<CropType, Double> cropAdjustments;
	private boolean calibrateToObserved;
	private Map<CropType, Double> tradeBarriers;
	
/*	private double maxLandUseChange;
	private double meatEfficiency;
	private double minFeedRate;
	private double landChangeEnergy;*/
	
	
	private GamsCountryInput(CompositeCountry country, Map<CommodityType, Double> projectedDemand, Map<CropType, Double> tradeBarriers, Map<CropType, GlobalPrice> worldPrices,
			Map<CropType, ImportExportConstraint> importConstraints, Map<CropType, Double> cropAdjustments, boolean calibrateToObserved) {
		super();
		this.country = country;
		this.projectedDemand = projectedDemand;
		this.worldPrices = worldPrices;
		this.importConstraints = importConstraints;
		this.cropAdjustments = cropAdjustments;
		this.calibrateToObserved = calibrateToObserved;
		this.tradeBarriers = tradeBarriers;
	}
	
	public GamsCountryInput(GamsCountryInput gamsInput, Map<CropType, Double> cropAdjs) {
		this(gamsInput.country, gamsInput.projectedDemand, gamsInput.tradeBarriers, gamsInput.worldPrices, gamsInput.importConstraints, cropAdjs, false);
	}
	
	public static GamsCountryInput createInput(CompositeCountry country, Map<CommodityType, Double> projectedDemand, Map<CropType, Double> tradeBarriers, Map<CropType, GlobalPrice> worldPrices,
			Map<CropType, Double> baseNetImport, Map<CropType, Double> maxOfProdOrSupply, Map<CropType, Double> cropAdjustments, boolean calibrateToObserved) {
			
		double allowedImportChange = calibrateToObserved ? 0.0 : ModelConfig.MAX_IMPORT_CHANGE;		
		Map<CropType, ImportExportConstraint> importConstraints = new HashMap<CropType, ImportExportConstraint>();
		
		for (Map.Entry<CropType, Double> entry : baseNetImport.entrySet()) {
			CropType c = entry.getKey();
			double change = allowedImportChange * maxOfProdOrSupply.get(c);
			importConstraints.put(c, new ImportExportConstraint(entry.getValue() - change, entry.getValue() + change));
		}

		return new GamsCountryInput(country, projectedDemand, tradeBarriers, worldPrices, importConstraints, cropAdjustments, calibrateToObserved);
	}
	

	public CompositeCountry getCountry() { 
		return country;
	}
	
	public Map<CommodityType, Double> getProjectedDemand() {
		return projectedDemand;
	}

	public Map<CropType, GlobalPrice> getWorldPrices() {
		return worldPrices;
	}
	
	public Map<CropType, ImportExportConstraint> getImportConstraints() {
		return importConstraints;
	}

//	public Map<CropType, Double> getMinFeedAmount() {
//		return minFeedAmount;
//	}

	public double getMeatEfficiency() {
		return ModelConfig.MEAT_EFFICIENCY;  // this is already handled by the feed conversion efficiency for each animal product
	}

	public double getLandChangeCost() {
		return ModelConfig.LAND_CHANGE_COST;
	}

	public double getMinFeedRate() {
		return ModelConfig.MIN_FEED_RATE;
	}
	
	public Map<CropType, Double> getCropAdjustments() {
		return cropAdjustments;
	}

	public boolean isCalibrateToObserved() {
		return calibrateToObserved;
	}

	public Map<CropType, Double> getMinNetImport() {
		Map<CropType, Double> netImport = new HashMap<CropType, Double>();
		for (Map.Entry<CropType, ImportExportConstraint> entry : importConstraints.entrySet()) {
			netImport.put(entry.getKey(), entry.getValue().getMinNetImport());
		}
		return netImport;
	}

	public Map<CropType, Double> getMaxNetImport() {
		Map<CropType, Double> netImport = new HashMap<CropType, Double>();
		for (Map.Entry<CropType, ImportExportConstraint> entry : importConstraints.entrySet()) {
			netImport.put(entry.getKey(), entry.getValue().getMaxNetImport());
		}
		return netImport;
	}

	public Map<CropType, Double> getCountryImportPrices() {
		double tradeBarrier;
		double baseImportPrice;
		Map<CropType, Double> prices = new HashMap<CropType, Double>();
		for (Map.Entry<CropType, GlobalPrice> entry : worldPrices.entrySet()) {
				
			tradeBarrier =  tradeBarriers.get(entry.getKey());
			baseImportPrice = entry.getValue().getImportPrice();
			prices.put(entry.getKey(), baseImportPrice+baseImportPrice*tradeBarrier);
		}
		return prices;
	}
	
	public Map<CropType, Double> getWorldExportPrices() {
		Map<CropType, Double> prices = new HashMap<CropType, Double>();
		
		for (Map.Entry<CropType, GlobalPrice> entry : worldPrices.entrySet()) {
			prices.put(entry.getKey(), entry.getValue().getExportPrice());
		}
		return prices;
	}
}

