package ac.ed.lurg.country.gams;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.landuse.IrrigationItem;
import ac.ed.lurg.landuse.LandUseItem;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.types.LandCoverType;
import ac.ed.lurg.types.YieldType;
import ac.ed.lurg.utils.LazyTreeMap;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.yield.YieldRaster;
import ac.ed.lurg.yield.YieldResponsesItem;
import ac.sac.raster.IntegerRasterItem;
import ac.sac.raster.RasterKey;
import ac.sac.raster.RasterSet;

public class GamsRasterOptimiser {
	public static final boolean DEBUG = false;

	private GamsRasterInput rasterInputData;
	private LazyTreeMap<Integer, Set<RasterKey>> mapping;

	public GamsRasterOptimiser(GamsRasterInput rasterInputData) {
		this.rasterInputData = rasterInputData;
	}

	public GamsRasterOutput run() {
		// workout similar areas		
		GamsLocationInput gamsInput = convertFromRaster(rasterInputData);

		long start = System.currentTimeMillis();

		GamsLocationOutput gamsOutput = runGamsWithCalib(gamsInput);

		long last = System.currentTimeMillis();
		LogWriter.printlnError("Took " + (last-start));

		// map results back to raster
		return convertToRaster(gamsInput, gamsOutput);
	}

	/** Probably should be in GAMS class */
	private GamsLocationOutput runGamsWithCalib(GamsLocationInput gamsInput) {

		GamsLocationOutput result = null;

		/*  This code used to try to do some calibration, but was causing more distortions than it fixed.
		if (gamsInput.getCountryInput().isCalibrateToObserved()) {
			Map<CropType, Double> cropAdjustments = new HashMap<CropType, Double>();  // recreate, but might have been null is this case anyway
			for (CropType c : CropType.getNonMeatTypes()) {
				cropAdjustments.put(c, 1.0);
			}

			double baseCropland = LandUseItem.getTotalLandCover(gamsInput.getPreviousLandUse().values(), LandCoverType.CROPLAND);
			double basePasture = LandUseItem.getTotalLandCover(gamsInput.getPreviousLandUse().values(), LandCoverType.PASTURE);

			for (int i=0; i<ModelConfig.NUM_CALIBRATION_ITERATIONS; i++) {

				GamsLocationInput newInput = GamsLocationInput.createWithNewAdjustments(gamsInput, cropAdjustments);
				GamsLocationOptimiser opti= new GamsLocationOptimiser(newInput);
				result = opti.run();

				if (baseCropland > 0) {
					double cropAdjSingle = LandUseItem.getTotalLandCover(result.getLandUses().values(), LandCoverType.CROPLAND) / baseCropland;
					cropAdjSingle = Math.min(Math.max(cropAdjSingle, 0.5), 2);

					for (CropType c : CropType.getCropsLessPasture()) {
						cropAdjustments.put(c, Math.min(Math.max(cropAdjustments.get(c) * cropAdjSingle, 0.2), 5));
					}
				}

				if (basePasture > 0) {
					double pastureAdjSingle = LandUseItem.getTotalLandCover(result.getLandUses().values(), LandCoverType.PASTURE) / basePasture;
					pastureAdjSingle = Math.min(Math.max(pastureAdjSingle, 0.5), 2);
					cropAdjustments.put(CropType.PASTURE, Math.min(Math.max(cropAdjustments.get(CropType.PASTURE) * pastureAdjSingle, 0.2), 5));
				}
			}
		}
		else { */
			GamsLocationOptimiser opti = new GamsLocationOptimiser(gamsInput);		
			result = opti.run();
	//	}
		return result;
	}


	private GamsRasterOutput convertToRaster(GamsLocationInput gamsInput, GamsLocationOutput gamsOutput) {		
		RasterSet<LandUseItem> newIntensityRaster = allocAreas(gamsInput.getPreviousLandUse(), gamsOutput);

		RasterSet<IntegerRasterItem> locationIdRaster = new RasterSet<IntegerRasterItem>(rasterInputData.getPreviousLandUses().getHeaderDetails());
		for (Entry<Integer, Set<RasterKey>> entry : mapping.entrySet()) {
			Integer locId = entry.getKey();
			for (RasterKey key : entry.getValue())
				locationIdRaster.put(key, new IntegerRasterItem(locId));
		}

		return new GamsRasterOutput(gamsOutput.getStatus(), newIntensityRaster, gamsOutput.getCommoditiesData(), gamsOutput.getCropAdjustments(), locationIdRaster);
	}

	private RasterSet<LandUseItem> createWithSameLandCovers(RasterSet<LandUseItem> toCopy) {
		RasterSet<LandUseItem> theCopy = new RasterSet<LandUseItem>(toCopy.getHeaderDetails());

		for (Entry<RasterKey, LandUseItem> entry : toCopy.entrySet()) {
			LandUseItem newAreasItem = new LandUseItem();
			newAreasItem.setLandCoverAreas(entry.getValue());
			theCopy.put(entry.getKey(), newAreasItem);
		}

		return theCopy;
	}

	private void checkedTotalAreas(RasterSet<LandUseItem> areaRaster, String comment) {
		for (LandCoverType l : LandCoverType.values()) {
			double total = 0;
			for (LandUseItem a : areaRaster.values()) {
				total += a.getLandCoverArea(l);
			}

			LogWriter.printlnError("Total Area " + comment + ": " + l.getName() + ": " + total);
		}
	}

	private RasterSet<LandUseItem> allocAreas(Map<Integer, ? extends LandUseItem> prevAreasAgg, GamsLocationOutput gamsOutput) {
		RasterSet<LandUseItem> newLandUseRaster = createWithSameLandCovers(rasterInputData.getPreviousLandUses());

		//checkedTotalAreas(rasterInputData.getPreviousAreas(), "old");
		//checkedTotalAreas(newAreaRaster, "new");

		for (Map.Entry<Integer, LandUseItem> entry : gamsOutput.getLandUses().entrySet()) {
			Integer locId = entry.getKey();
			LandUseItem newLandUseAggItem = entry.getValue();
			LandUseItem prevLandUseAggItem = prevAreasAgg.get(locId);
			Set<RasterKey> keys = mapping.get(locId);

			if (DEBUG) 
				checkedTotalAreas(newLandUseRaster.createSubsetForKeys(keys), locId + " before");

			double pastureChange = newLandUseAggItem.getLandCoverArea(LandCoverType.PASTURE) - prevLandUseAggItem.getLandCoverArea(LandCoverType.PASTURE);
			double croplandChange = newLandUseAggItem.getLandCoverArea(LandCoverType.CROPLAND) - prevLandUseAggItem.getLandCoverArea(LandCoverType.CROPLAND);
			double prevForestToNaturalFraction = prevLandUseAggItem.getForestToNaturalFraction();

			double pastureFromCrop = 0;
			double pastureFromNatural = 0;
			double cropFromNatural = 0;

			if (pastureChange > 0 && croplandChange < 0) {
				pastureFromCrop = Math.min(pastureChange, -croplandChange);

				if (pastureChange > -croplandChange)
					pastureFromNatural = pastureChange+croplandChange;
				else
					cropFromNatural = pastureChange+croplandChange;
			}
			else if (pastureChange < 0 && croplandChange > 0) {
				pastureFromCrop = -Math.min(-pastureChange, croplandChange);

				if (croplandChange > -pastureChange)
					cropFromNatural = pastureChange+croplandChange;
				else
					pastureFromNatural = pastureChange+croplandChange;
			}
			else {
				pastureFromNatural = pastureChange;
				cropFromNatural = croplandChange;
			}

			double shortfall = 0;
			shortfall += allocAllLandCrops(newLandUseRaster, keys, LandCoverType.PASTURE, LandCoverType.FOREST, pastureFromNatural * prevForestToNaturalFraction);
			shortfall += allocAllLandCrops(newLandUseRaster, keys, LandCoverType.PASTURE, LandCoverType.OTHER_NATURAL, pastureFromNatural * (1-prevForestToNaturalFraction));
			shortfall += allocAllLandCrops(newLandUseRaster, keys, LandCoverType.PASTURE, LandCoverType.CROPLAND, pastureFromCrop);
			shortfall += allocAllLandCrops(newLandUseRaster, keys, LandCoverType.CROPLAND, LandCoverType.FOREST, cropFromNatural * prevForestToNaturalFraction);
			shortfall += allocAllLandCrops(newLandUseRaster, keys, LandCoverType.CROPLAND, LandCoverType.OTHER_NATURAL, cropFromNatural * (1-prevForestToNaturalFraction));

			if (shortfall > 0.00001)
				LogWriter.printlnError("This should never happen, due to GAMS constraint. Not able to incorporate all changes, as not enough forest or natural areas left: " + locId + ": " + shortfall);

			if (DEBUG) 
				checkedTotalAreas(newLandUseRaster.createSubsetForKeys(keys), locId + " after");

			for (RasterKey key : keys) {
				LandUseItem newLandUseItem = newLandUseRaster.get(key);
				for (CropType crop : CropType.values()) {
					newLandUseItem.setCropFraction(crop, newLandUseAggItem.getCropFraction(crop));
					newLandUseItem.setIntensity(crop, newLandUseAggItem.getIntensity(crop)); // intensities constant over single aggregated land category
				}
			}
		}

		return newLandUseRaster;
	}

	private double allocAllLandCrops(RasterSet<LandUseItem> newLandUseRaster, Set<RasterKey> keys, LandCoverType toLC, LandCoverType fromLC, double change) {
		LandCoverType toType = toLC;
		LandCoverType fromType = fromLC;

		// reverse direction if negative
		if (change < 0) {
			change = -change;
			toType = fromLC;
			fromType = toLC;
		}

		double totalShortfall = 0;
		Set<RasterKey> keysWithSpace = new HashSet<RasterKey>();
		double avgChange = change/keys.size();

		for (RasterKey key : keys) {
			//if (DEBUG) LogWriter.println("  Processing raster key " + key);
			LandUseItem newLandUseItem = newLandUseRaster.get(key);

			double shortfall = newLandUseItem.moveAreas(toType, fromType, avgChange);
			if (shortfall == 0)
				keysWithSpace.add(key);
			else
				totalShortfall += shortfall;
		}

		if (totalShortfall > 0 & keysWithSpace.size() > 0) {  // more to allocate and some free areas to allocate into
			if (DEBUG) LogWriter.println("A total land cover of " + totalShortfall + " is unallocated, so trying again with areas with space");
			totalShortfall = allocAllLandCrops(newLandUseRaster, keysWithSpace, toType, fromType, totalShortfall);
		}

		return totalShortfall;
	}

	@SuppressWarnings("serial")
	private GamsLocationInput convertFromRaster(GamsRasterInput rasterInputData) {
		// as a first attempt only going to look at pasture and cereal yields, assuming other yields will be correlated to one or the other.
		YieldRaster yieldRaster = rasterInputData.getYields();
		RasterSet<LandUseItem> landUseRaster = rasterInputData.getPreviousLandUses();

		/*	checkedTotalAreas(cropAreaRaster, "before");
		new RasterOutputer<AreasItem>(cropAreaRaster, "BeforeLandCover") {
			@Override
			public Double getValue(RasterKey location) {
				AreasItem area = results.get(location);
				if (area == null)
					return null;

				int count = 0;
				double maxLC = 0;
				int maxId = 0;
				for (LandCoverType l : LandCoverType.values()) {
					count++;
					if (maxLC < area.getLandCoverArea(l)) {
						maxLC = area.getLandCoverArea(l);
						maxId = count;
					}
				}
				return (double)maxId;
			}

			@Override
			public int convertToPixelValue(double value) {
				return (int) (1 + value * 2);
			}
		}.writeOutput(true); */


		RasterSet<IrrigationItem> irrigRaster = rasterInputData.getIrrigationData();

		{
			//	YieldResponsesItem yresp = yieldRaster.get(new RasterKey(40,38));//getFromCoordinates(-118.0, -33.0);
			//	LogWriter.printlnError("Test key2: " + yresp.getYieldMax(CropType.WHEAT)  + ", " + yresp.getYieldFertOnly(CropType.WHEAT) + ", " + yresp.getYieldIrrigOnly(CropType.WHEAT));
		}

		// look for inconsistencies
		for (Entry<RasterKey, YieldResponsesItem> entry : yieldRaster.entrySet()) {

			YieldResponsesItem yresp = entry.getValue();
			RasterKey key = entry.getKey();
			CropType crop = CropType.WHEAT;

			if (yresp != null && (yresp.getYieldMax(crop) < yresp.getYieldFertOnly(crop) || yresp.getYieldMax(crop) < yresp.getYieldIrrigOnly(crop))) {
				logErrorWithCoord("Inconsistency F only:" + yresp.getYieldFertOnly(crop) + ", I only" + yresp.getYieldIrrigOnly(crop) + ", max " + yresp.getYieldMax(crop) + " at ", key, yieldRaster);
			}
		}

		int numCerealCats = ModelConfig.NUM_CEREAL_CATEGORIES;
		int numPastureCats = ModelConfig.NUM_PASTURE_CATEGORIES;

		int thisShouldLookAtCropsOtherThanJustWheat; // need to consider other crops, and perhaps other yieldTypes as well - particularly fert/irrig responses
		List<Double> wheatlDivisions = getDivisions(yieldRaster, CropType.WHEAT, numCerealCats); 
		List<Double> pastureDivisions = getDivisions(yieldRaster, CropType.PASTURE, numPastureCats);

		if (DEBUG) LogWriter.println("Making " + numCerealCats * numPastureCats + " categories");

		LazyTreeMap<Integer, YieldResponsesItem> aggregatedYields = new LazyTreeMap<Integer, YieldResponsesItem>() { 
			protected YieldResponsesItem createValue() { return new YieldResponsesItem(); }
		};
		LazyTreeMap<Integer, LandUseItem> aggregatedAreas = new LazyTreeMap<Integer, LandUseItem>() { 
			protected LandUseItem createValue() { return new LandUseItem(); }
		};
		LazyTreeMap<Integer, IrrigationItem> aggregatedIrrigCosts = new LazyTreeMap<Integer, IrrigationItem>() { 
			protected IrrigationItem createValue() { return new IrrigationItem(); }
		};
		mapping = new LazyTreeMap<Integer, Set<RasterKey>>() { 
			protected Set<RasterKey> createValue() { return new HashSet<RasterKey>(); }
		};

		int countFound = 0, countMissing = 0;

		for (Entry<RasterKey, YieldResponsesItem> entry : yieldRaster.entrySet()) {
			YieldResponsesItem yresp = entry.getValue();
			RasterKey key = entry.getKey();

			if (yresp == null) {
				countMissing++;
				logErrorWithCoord("No YieldResponsesItem for ", key, yieldRaster);
			}
			else {
				countFound++;
				//logErrorWithCoord("YieldResponsesItem found for ", key, yieldRaster);

				LandUseItem landUseItem  = landUseRaster.get(key);
				if (landUseItem == null) {
					LogWriter.printlnError("GamsRasterOptimiser: Can't find cropAreas for " + key);
					continue;
				}

				IrrigationItem irrigItem = irrigRaster.get(key);

				int cerealCat = findCategory(wheatlDivisions, yresp.getYieldNone(CropType.WHEAT) + yresp.getYieldMax(CropType.WHEAT));
				int pastureCat = findCategory(pastureDivisions, yresp.getYieldNone(CropType.PASTURE) + yresp.getYieldMax(CropType.PASTURE));
				Integer id = cerealCat + pastureCat * numCerealCats;

				YieldResponsesItem aggYResp = aggregatedYields.lazyGet(id);
				LandUseItem aggLandUse = aggregatedAreas.lazyGet(id);
				IrrigationItem aggIrig = aggregatedIrrigCosts.lazyGet(id);
				mapping.lazyGet(id).add(key); 

				// Irrigation cost
				double suitableAreaThisTime  = landUseItem.getSuitableLand();
				double suitableAreaSoFar = aggLandUse.getSuitableLand();
				if (irrigItem!= null) {
					aggIrig.setIrrigCost( aggregateMean(aggIrig.getIrrigCost(), suitableAreaSoFar, irrigItem.getIrrigCost(), suitableAreaThisTime));
					aggIrig.setIrrigConstraint(aggregateMean(aggIrig.getIrrigConstraint(), suitableAreaSoFar, irrigItem.getIrrigConstraint(), suitableAreaThisTime));
					//LogWriter.println(id + ", " + key + ", " + avgIrigCost.getIrrigCost() + ", from " + areaSoFar + ", " + suitableAreaThisTime + ", " + irrigCost.getIrrigCost());
				}

				// Crops yields and area fractions
				for (CropType crop : CropType.getNonMeatTypes()) {
					if (irrigItem!= null) {							
						double irrigMax = irrigItem.getMaxIrrigAmount(crop);

						if (Double.isNaN(irrigMax))
							logErrorWithCoord("Can't find irrig max amount for ", key, yieldRaster, crop);
						else 
							aggIrig.setMaxIrrigAmount(crop, aggregateMean(aggIrig.getMaxIrrigAmount(crop), suitableAreaSoFar, irrigMax, suitableAreaThisTime));
					}
					
					for (YieldType yieldType : YieldType.values()) {
						double y = yresp.getYield(yieldType, crop);
						if (Double.isNaN(y))
							logErrorWithCoord("Problem getting yield for type:" + yieldType + ", ", key, yieldRaster, crop);
						else {
							double yieldSoFar = aggYResp.getYield(yieldType, crop);
							double updatedYield = aggregateMean(yieldSoFar, suitableAreaSoFar, y, suitableAreaThisTime);
							aggYResp.setYield(yieldType, crop, updatedYield);
						}
					}
				
					double areaSoFar = aggLandUse.getCropArea(crop);
					double areaThisTime = landUseItem.getCropArea(crop);
					double updateCropland = (landUseItem.getLandCoverArea(LandCoverType.CROPLAND) + aggLandUse.getLandCoverArea(LandCoverType.CROPLAND));

					if (updateCropland > 0)
						aggLandUse.setCropFraction(crop, (areaSoFar + areaThisTime) / updateCropland);
				}

				// Land covers ares
				for (LandCoverType landType : LandCoverType.values()) {
					double areaThisTime = landUseItem.getLandCoverArea(landType);
					double areaSoFar = aggLandUse.getLandCoverArea(landType);
					aggLandUse.setLandCoverArea(landType, areaSoFar + areaThisTime);
				}
			}
		}

		LogWriter.println("YieldResponsesItem: " + rasterInputData.getCountryInput().getCountry() + ", countFound=" + countFound + ", countMissing=" + countMissing);

		for (Map.Entry<Integer, Set<RasterKey>> e : mapping.entrySet()) {
			LogWriter.println(e.getKey() + " zone has " + e.getValue().size() + " raster areas");

			/*	CropType[] cs = {CropType.WHEAT, CropType.MAIZE};
			for (CropType c : cs) {
				for (RasterKey key : e.getValue()) {
					YieldResponsesItem yri = yieldRaster.get(key);
					LogWriter.println(String.format("\t%s: x=%.1f y=%.1f: %s yields (none,fmid,fmax,irrig,both) %.1f, %.1f, %.1f, %.1f, %.1f", 
							key, yieldRaster.getXCoordin(key), yieldRaster.getYCoordin(key), c.getGamsName(),
							yri.getYieldNone(c), yri.getYieldMidFertOnly(c), yri.getYieldFertOnly(c), yri.getYieldIrrigOnly(c), yri.getYieldMax(c)));
				}
				LogWriter.println("");
			} */
		}
		
		double baseCropland = LandUseItem.getTotalLandCover(aggregatedAreas.values(), LandCoverType.CROPLAND);
		double basePasture = LandUseItem.getTotalLandCover(aggregatedAreas.values(), LandCoverType.PASTURE);
		LogWriter.println("\nbaseCropland=" + baseCropland + ", basePasture=" + basePasture);

		return new GamsLocationInput(rasterInputData.getTimestep(), aggregatedYields, aggregatedAreas, aggregatedIrrigCosts, rasterInputData.getCountryInput());
	}
	
	private void logErrorWithCoord(String message, RasterKey key, YieldRaster yieldRaster, CropType crop) {
		logErrorWithCoord(message + "crop:" + crop + ", ", key, yieldRaster);
	}
	
	private void logErrorWithCoord(String message, RasterKey key, YieldRaster yieldRaster) {
		LogWriter.printlnError(message + key + ", x:" + yieldRaster.getXCoordin(key) + ", y:" + yieldRaster.getYCoordin(key));
	}
	
	private double aggregateMean(double aggV, double aggArea, double newV, double newArea) {
		if (Double.isNaN(aggV))
			return newV;
		
		if (newArea == 0)
			return aggV;
		
		return (aggV*aggArea + newV*newArea) / (aggArea + newArea);
	}

	private int findCategory(List<Double> divisions, double yield) {
		int category;
		int numDivisions = divisions.size();

		for (category = 0; category<numDivisions; category++) {
			if (yield < divisions.get(category)) {
				break;
			}
		}
		return category;
	}

	private List<Double> getDivisions(YieldRaster yieldRaster, CropType crop, int numCategories) {
		List<Double> yieldValues = new ArrayList<Double>();

		for (YieldResponsesItem yresp : yieldRaster.values()) {
			if (yresp == null) {
				if (DEBUG) LogWriter.println("GamsRasterOptimiser: Can't get value for crop " + crop);
			}
			else {
				double d = yresp.getYieldNone(crop) + yresp.getYieldMax(crop) ;
				//LogWriter.println("GamsRasterOptimiser: Got value for crop " + crop + " of " + d);

				if (Double.isNaN(d) || d == 0.0) {
					if (DEBUG) LogWriter.println("GamsRasterOptimiser: Got NaN or zero value for crop " + crop);
				}
				else {
					yieldValues.add(d);
				}
			}
		}

		if (yieldValues.size() == 0) {
			throw new RuntimeException("No yield values for country, crop = " + crop);
		}

		Collections.sort(yieldValues);

		List<Double> divisions = new ArrayList<Double>();

		for (int i=1; i < numCategories; i++) {
			double d = yieldValues.get(yieldValues.size()*i/numCategories);
			divisions.add(d);
		}	

		return divisions;
	}
}
