package ac.ed.lurg.demand;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LogWriter;

public class BaseConsumpManager {
	private static final int COUNTRY_COL = 0; 
	private static final int BASE_POP_COL = 1; 
	private static final int COMMODITY_COL = 2; 
	private static final int BASE_CPC_COL = 3; 
	
	private Map<SingleCountry, Map<CommodityType, Double>> baseConsumpMap = new HashMap<SingleCountry, Map<CommodityType, Double>>();
	// Don't like the duplication of population in this base consumption file, but guess not really a problem
	private Map<SingleCountry, Double> basePop = new HashMap<SingleCountry, Double>();

	public BaseConsumpManager() {
		super();
		read();
	}
	
	public void read() {
		String filename = ModelConfig.BASELINE_CONSUMP_FILE;
		try {
			BufferedReader fitReader = new BufferedReader(new FileReader(filename)); 
			String line, countryName, commodityName;
			double pop, cpc;
			fitReader.readLine(); // read header

			while ((line=fitReader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 4)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				countryName = tokens[COUNTRY_COL];
				pop = Double.parseDouble(tokens[BASE_POP_COL]);
				commodityName = tokens[COMMODITY_COL];
				cpc = Double.parseDouble(tokens[BASE_CPC_COL]);

				SingleCountry country = CountryManager.getForName(countryName);
				CommodityType commodity = CommodityType.getForFaoName(commodityName);
				
				Map<CommodityType, Double> commodityMap = baseConsumpMap.get(country);
				if (commodityMap == null) {
					commodityMap = new HashMap<CommodityType, Double>();
					baseConsumpMap.put(country, commodityMap);
				}
				
				commodityMap.put(commodity, cpc);
				basePop.put(country, pop);
			} 
			fitReader.close(); 
		
		} catch (Exception e) {
			LogWriter.printlnError("Failed in reading commodity demand fits");
			LogWriter.print(e);
		}
		LogWriter.println("Processed " + filename + ", create " + baseConsumpMap.size() + " base consumption values");
	}
	
	public double get(SingleCountry country, CommodityType commodity) {
		Map<CommodityType, Double> commodityMap = baseConsumpMap.get(country);

		if (commodityMap == null || !commodityMap.containsKey(commodity)) {
			LogWriter.printlnError("BaseConsumpManager: can't get value for " + commodity + ", " + country);
			return 0.0;
		}
		
		Double d = commodityMap.get(commodity);
		return d.doubleValue();
	}
	
	public double getBasePopulation(SingleCountry country) {
		Double d = basePop.get(country);
		return (d != null) ? d.doubleValue() : Double.NaN;
	}
	
	public Collection<SingleCountry> getAllCountries() {
		return basePop.keySet();
	}
}
