package ac.ed.lurg.demand;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LazyHashMap;
import ac.ed.lurg.utils.LogWriter;

public class BioenergyDemand {
	private static final int COUNTRY_COL = 0; 
	private static final int COMMODITY_COL = 2; 
	private static final int OTHER_COL = 3; 
	
	@SuppressWarnings("serial")
	public Map<SingleCountry, Map<CommodityType, Double>> getBioenergyData() {
		
		LazyHashMap<SingleCountry, Map<CommodityType, Double>> bioenergyMap = new LazyHashMap<SingleCountry, Map<CommodityType, Double>>() {
			protected Map<CommodityType, Double> createValue() { return new HashMap<CommodityType, Double>(); }
		};
		
		String filename = ModelConfig.BIOENERGY_DEMAND_FILE;
		try {
			BufferedReader fitReader = new BufferedReader(new FileReader(filename)); 
			String line, countryName, commodityName;
			double other;
			fitReader.readLine(); // read header

			while ((line=fitReader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 4)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				countryName = tokens[COUNTRY_COL];
				commodityName = tokens[COMMODITY_COL];
				other = Double.parseDouble(tokens[OTHER_COL]);

				SingleCountry country = CountryManager.getForName(countryName);
				CommodityType crop = CommodityType.getForFaoName(commodityName);
				
				Map<CommodityType, Double> countryData = bioenergyMap.lazyGet(country);
				countryData.put(crop, other);
			} 
			fitReader.close(); 
		
		} catch (IOException e) {
			LogWriter.printlnError("Failed in reading commodity demand fits");
			LogWriter.print(e);
		}
		LogWriter.println("Processed " + filename + ", create " + bioenergyMap.size() + " country commodity maps values");
		
		return bioenergyMap;
	}
}

