package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.ModelFitType;
import ac.ed.lurg.utils.LogWriter;

public class DemandManager {

	private DemandCurveManager demandCurveManager;
	private SspManager sspManager;
	private BaseConsumpManager baseConsumpManager;
	private CompositeCountryManager compositeCountryManager;
	private Map<SingleCountry, Map<CommodityType, Double>> bioenergyBaseDemand;

	private ModelFitType fitType;
	private String ssp_scenario;

	public DemandManager (ModelFitType fitType, String ssp_scenario, BaseConsumpManager baseConsumpManager, CompositeCountryManager compositeCountryManager) {
		this.fitType = fitType;
		this.ssp_scenario = ssp_scenario;
		this.baseConsumpManager = baseConsumpManager;
		this.compositeCountryManager = compositeCountryManager;
		demandCurveManager = new DemandCurveManager();
		sspManager = new SspManager();
		bioenergyBaseDemand = new BioenergyDemand().getBioenergyData();
	}

	public Map<CommodityType, Double> getDemand(CompositeCountry cc, int year) {

		if (!ModelConfig.CHANGE_DEMAND_YEAR) 
			year = ModelConfig.BASE_YEAR;

		Map<CommodityType, Double> demandMap = new HashMap<CommodityType, Double>();

		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {
			SspData baseSspData = sspManager.get(ssp_scenario, ModelConfig.BASE_YEAR, c);

			SspData sd = sspManager.get(ssp_scenario, year, c);

			LogWriter.println("Got ssp data for " + c.getCountryName() + " of " + sd);

			if (sd == null) {
				LogWriter.printlnError("No ssp data for " + ssp_scenario + ", " + year + ", " + c.getCountryName());
				continue;
			}

			for (DemandCurve dc : demandCurveManager.get(fitType)) {
				double cpc = dc.getConsumptionPc(baseSspData.getGdpPc(), baseConsumpManager.get(c, dc.getCommodityType()), sd.getGdpPc());
				double d = cpc * sd.getPopulation();

				double bioenergy = getBioenergyDemand(c, year, dc.getCommodityType());
				//		LogWriter.println(String.format("Country %s comm %s: %f",  country, dc.getCommodityType(), bioenergy));

				double newDemand = (d + bioenergy) * (1.0 + ModelConfig.SEED_AND_WASTE_FRACTION);
				Double prevDemand = demandMap.get(dc.getCommodityType());

				if (prevDemand != null)
					newDemand += prevDemand.doubleValue();  // aggregate if composite country

				demandMap.put(dc.getCommodityType(), newDemand);
			}
		}

		return demandMap;
	}

	private double getBioenergyDemand(SingleCountry country, int year, CommodityType commodity) {
		// could adjust for year somehow, but not doing this yet
		if (bioenergyBaseDemand != null && bioenergyBaseDemand.containsKey(country)) {
			Double d = bioenergyBaseDemand.get(country).get(commodity);
			return d == null ? 0 : d.doubleValue();
		}
		return 0.0;
	}

	/*	public double getPopulation (SingleCountry country, int year) {
		SspData sd = sspManager.get(ssp_scenario, year, country);
		if (sd == null) {
			LogWriter.printlnError("No ssp data for " + ssp_scenario + ", " + year + ", " + country.getCountryName());
			return 0;
		}
		else
			return sd.getPopulation();
	} 

	public String getModelAndScenarioDescription() {
		return fitType + " " + ssp_scenario;
	}
	 */
}
