package ac.ed.lurg.landuse;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.CountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LazyHashMap;
import ac.ed.lurg.utils.LogWriter;

public class CropUsageReader {
	private static final int COUNTRY_COL = 0; 
	private static final int COMMODITY_COL = 2; 
	private static final int FEED_COL = 4; 
	private static final int NET_IMPORT_COL = 5; 
	private static final int PROD_COL = 6; 
	
	private CompositeCountryManager compositeCountryManager;
	
	public CropUsageReader(CompositeCountryManager compositeCountryManager) {
		this.compositeCountryManager = compositeCountryManager;
	}

	@SuppressWarnings("serial")
	public Map<CompositeCountry, Map<CropType, CropUsageData>> getCommodityData() {
		
		LazyHashMap<CompositeCountry, Map<CropType, CropUsageData>> commodityMap = new LazyHashMap<CompositeCountry, Map<CropType, CropUsageData>>() {
			protected Map<CropType, CropUsageData> createValue() { return new HashMap<CropType, CropUsageData>(); }
		};
		
		String filename = ModelConfig.COMMODITY_DATA_FILE;
		try {
			BufferedReader fitReader = new BufferedReader(new FileReader(filename)); 
			String line, countryName, commodityName;
			double feedAmount, netImports, prod;
			fitReader.readLine(); // read header

			while ((line=fitReader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 6)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				countryName = tokens[COUNTRY_COL];
				commodityName = tokens[COMMODITY_COL];
				feedAmount = Double.parseDouble(tokens[FEED_COL]);
				netImports = Double.parseDouble(tokens[NET_IMPORT_COL]);
				prod = Double.parseDouble(tokens[PROD_COL]);

				SingleCountry country = CountryManager.getForName(countryName);
				
				CropType crop = CropType.getForFaoName(commodityName);
				CompositeCountry cc = compositeCountryManager.getForSingleCountry(country);
				
				Map<CropType, CropUsageData> countryData = commodityMap.lazyGet(cc);
				CropUsageData oldData = countryData.get(crop);
				
				// aggregate if multiple countries for cc
				if (oldData != null) {
					feedAmount += oldData.getFeedAmount();
					netImports += oldData.getNetImports();
					prod += oldData.getProduction();
				}
				
				CropUsageData newData = new CropUsageData(feedAmount, netImports, prod, Double.NaN);
				countryData.put(crop, newData);
			} 
			fitReader.close(); 
		
		} catch (IOException e) {
			LogWriter.printlnError("Failed in reading commodity demand fits");
			LogWriter.print(e);
		}
		LogWriter.println("Processed " + filename + ", create " + commodityMap.size() + " country commodity maps values");
		
		return commodityMap;
	}
}
