package ac.ed.lurg.landuse;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.types.CropType;
import ac.sac.raster.RasterItem;

public class IrrigationItem implements RasterItem {
	// we have gone from CGIAR Aridity Index to an aridity cost by doing the following:  1 - log10(aridityIndex+1)/5
	// when is the limited to be >= 0 (only Greenland had negative values)
	// this gives a value from 1 to 0, with 1 being the most costly and 0 the least.
	
	private double irrigationCost; // cost per 10000 Mlitre or 10^10 l or 10^7 m3 or 10^-2 km3, when multipled by rates from areas in Mha makes sense to have it in this apparently odd unit
	private double irrigConstraint;  // mm or litre/m2 (they are the same) of water available
	private Map<CropType, Double> maxIrrigAmounts = new HashMap<CropType, Double>(); // amount of water (litres/m2) used at maximum irrigation, i.e. to achieve no water stress

	public double getIrrigCost() {
		return irrigationCost;
	}
	
	public void setIrrigCost(double aridityIndex) {
		this.irrigationCost = aridityIndex;
	}
	
	public double getMaxIrrigAmount(CropType crop) {
		if (CropType.PASTURE == crop)
			return ModelConfig.PASTURE_MAX_IRRIGATION_RATE;

		Double d = maxIrrigAmounts.get(crop);
		return d==null ? Double.NaN : d.doubleValue();
	}
	
	public void setMaxIrrigAmount(CropType crop, double maxIrrig) {
		maxIrrigAmounts.put(crop, maxIrrig);
	}
	
	public double getIrrigConstraint() {
		return irrigConstraint;
	}
	
	public void setIrrigConstraint(double irrigConstraint) {
		this.irrigConstraint = irrigConstraint;
	}
}
