package ac.ed.lurg.landuse;

import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.types.FertiliserRate;
import ac.ed.lurg.types.IrrigationRate;
import ac.ed.lurg.types.YieldType;
import ac.ed.lurg.yield.LPJCropTypes;
import ac.ed.lurg.yield.YieldRaster;
import ac.sac.raster.AbstractTabularRasterReader;
import ac.sac.raster.RasterKey;
import ac.sac.raster.RasterSet;

public class IrrigationMaxAmountReader extends AbstractTabularRasterReader<IrrigationItem> {	

	private static final int MIN_COLS = 6;
	private YieldRaster yieldSurfaces;

	public IrrigationMaxAmountReader(RasterSet<IrrigationItem> irigCosts, YieldRaster yieldSurfaces) {
		super("\\s+", MIN_COLS, irigCosts);
		this.yieldSurfaces = yieldSurfaces;
	}
		
	@Override
	protected void setData(RasterKey key, IrrigationItem item, Map<String, Double> rowValues) {
		
		boolean isSpringWheat = yieldSurfaces.isSpringWheat(key);
		String highFertIrrigString = YieldType.getYieldType(FertiliserRate.MAX_FERT, IrrigationRate.MAX_IRRIG).getFertIrrigString();

		double ww = getValueForCol(rowValues, LPJCropTypes.WINTER_WHEAT + highFertIrrigString) * ModelConfig.IRRIGIATION_EFFICIENCY;
		double sw = getValueForCol(rowValues, LPJCropTypes.SPRING_WHEAT + highFertIrrigString) * ModelConfig.IRRIGIATION_EFFICIENCY;
		double co = getValueForCol(rowValues, LPJCropTypes.MAIZE + highFertIrrigString) * ModelConfig.IRRIGIATION_EFFICIENCY;
		double ri = getValueForCol(rowValues, LPJCropTypes.RICE + highFertIrrigString) * ModelConfig.IRRIGIATION_EFFICIENCY;
		
		item.setMaxIrrigAmount(CropType.WHEAT, isSpringWheat ? sw : ww);
		item.setMaxIrrigAmount(CropType.MAIZE, co);
		item.setMaxIrrigAmount(CropType.RICE, ri);
		item.setMaxIrrigAmount(CropType.OILCROPS, isSpringWheat ? sw : ww);
		item.setMaxIrrigAmount(CropType.PULSES, sw);
		item.setMaxIrrigAmount(CropType.STARCHY_ROOTS, sw);
	}
}
